local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage", "GatewayManage")

if ("POST" == request_method) then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400)
    end

    pk = args["ProductKey"]
    dn = args["DeviceName"]
    service_name = args["FactorId"]
    params = args["Params"]

    utils.check_pk(pk)
    utils.check_dn(dn)

    if (not (service_name and params)) then
        utils.error_exit(400, "invalid parameter")
    end

    ok, data = utils_device.call_service(pk, dn, service_name, params)
    if (not ok) then
        utils.error_exit(500)
    end
    
    if (data) then
        res_raw = json.encode(data)
    else
        res_raw = "{}"
    end
    
    utils.error_exit(200, res_raw)
end

utils.error_exit(400, "unsupported request method")
